# Loading module:
# 1. archive.yml
# 2. recipe.yml
# 3. Load_decadal.R (V)
setwd('/esarchive/scratch/aho/git/auto-s2s/')

## TODO: remove paths to personal scratchs 
source("/esarchive/scratch/vagudets/repos/csoperational/R/get_regrid_params.R")
source("/esarchive/scratch/vagudets/repos/cstools/R/s2dv_cube.R")
# Load required libraries/funs
source("modules/data_load/dates2load.R")
source("tools/libs.R")

## TODO: Move to CSOperational
# Conversion from startR_array to s2dv_array
#-------------------------------------------------------------------
attr2array <- function(attr){
  return(array(as.vector(attr), dim(attr)))
}

startR_to_s2dv <- function(startR_array){

  dates_dims <- dim(Subset(startR_array, 
                           along=c('dat','var',
                           'latitude', 'longitude', 'ensemble'), 
                           list(1,1,1,1,1), drop="selected")) 

  #sdates_dims <- dim(Subset(startR_array, 
  #                         along=c('dat','var','time','sweek','sday',
  #                         'latitude', 'longitude', 'ensemble'), 
  #                         list(1,1,1,1,1,1,1,1), drop="selected")) 

  dates_start <- attr(startR_array, 'Variables')$common$time
  dates_end <- attr(startR_array, 'Variables')$common$time
  #sdates <-  unlist(attr(startR_array, 'FileSelectors')$dat1$file_date)

  dim(dates_start) <- dates_dims
  dim(dates_end) <- dates_dims
  #dim(sdates) <- sdates_dims

  ## TODO: change this line when time attributes work correctly?
  time_dims <- c("time", "sday", "sweek", "syear", "fcst_syear")

  s2dv_object <- s2dv_cube(data = attr2array(startR_array), 
                           lon = attr2array(attr(startR_array,
                                            'Variables')$common$longitude),
                           lat = attr2array(attr(startR_array,
                                            'Variables')$common$latitude),
                           Variable = list(varName = names(attr(startR_array, 'Variables')$common)[4],
                                           level = NULL),
                           Dates = list(start = dates_start,
                                        end = dates_end),
                                        #sdate = sdates),
			               time_dims = time_dims,
                           when = Sys.time(),
                           source_files = attr2array(attr(startR_array, "Files"))
          #Datasets=list(exp1=list(InitializationsDates=list(Member_1="01011990", 
          #                                                  Members="Member_1")))
                          ) 
  return(s2dv_object)
}

#====================================================================
#====================================================================

# recipe_file <- "modules/data_load/testing_recipes/recipe_decadal.yml"
# recipe_file <- "modules/data_load/testing_recipes/recipe_decadal_daily.yml"

load_datasets <- function(recipe_file) {

  recipe <- read_yaml(recipe_file)
  recipe$filename <- recipe_file 
  archive <- read_yaml(paste0(recipe$Run$code_dir, "conf/archive_decadal.yml"))$archive


  #-------------------------
  # Read from recipe:
  #-------------------------
  exp.name <- recipe$Analysis$Datasets$System$name #'HadGEM3'
  ref.name <- recipe$Analysis$Datasets$Reference$name #'era5'
  member <- strsplit(recipe$Analysis$Datasets$System$member, ',')[[1]] #c("r1i1p1f2", "r2i1p1f2")
  variable <- recipe$Analysis$Variables$name #'tas'
  store.freq <- recipe$Analysis$Variables$freq #monthly_mean
  lats.min <- as.numeric(recipe$Analysis$Region$latmin) #0
  lats.max <- as.numeric(recipe$Analysis$Region$latmax) #10
  lons.min <- as.numeric(recipe$Analysis$Region$lonmin) #0
  lons.max <- as.numeric(recipe$Analysis$Region$lonmax) #10

  # change to: sdates <- dates2load(recipe, logger)
  sdates <- as.numeric(recipe$Analysis$Time$sdate$start):as.numeric(recipe$Analysis$Time$sdate$end) #1960:2015
#  fyear <- as.numeric(recipe$Analysis$Time$fyear$start):as.numeric(recipe$Analysis$Time$fyear$end) #1:1

  # change to: 
#  idxs <- NULL
#  idxs$hcst <- get_timeidx(sdates$hcst,
#                           recipe$Analysis$Time$leadtimemin,
#                           recipe$Analysis$Time$leadtimemax,
#                           time_freq=store.freq)
  time <- as.numeric(recipe$Analysis$Time$leadtimemin):as.numeric(recipe$Analysis$Time$leadtimemax)
  #TODO: daily data
  # If daily data & time is "month", need a function to calculate the indices. 
  # E.g., time = 1:4 means Dec to March, then indices will be c(31:(31+30+31+28+31)) for non-leap yr and c(31:(31+30+31+28), (31+30+31+28+2):(31+30+31+28+2+30)) for leap yr.
  if (store.freq == "daily_mean") {
    # for 1993-1994, Dec to March
    time <- c(31:(31+31+31+28+30))
  }

  season <- recipe$Analysis$Time$season

  #-------------------------
  # Read from archive:
  #-------------------------
  if (store.freq == "monthly_mean") {
    table <- archive$System[[exp.name]][[store.freq]]$table[[variable]]  #'Amon'
  } else {
    table <- 'day'
  }
  grid <- archive$System[[exp.name]][[store.freq]]$grid[[variable]]
  version <- archive$System[[exp.name]][[store.freq]]$version[[variable]]
  if (identical(member, 'all')) {
    member <- strsplit(archive$System[[exp.name]]$member, ',')[[1]]
  }

  #-------------------------
  # derived from above:
  #-------------------------
  if (lon_min >= 0) {
    circularsort <- CircularSort(0, 360)
  } else {
    circularsort <- CircularSort(-180, 180)
  }
  # generate transform params for system and ref
  regrid_params <- get_regrid_params(recipe, archive)

  # Only if the time length in each chunk may differ that we need largest_dims_length to be TRUE. Otherwise, set FALSE to increase efficiency.
  need_largest_dims_length <- ifelse(exp.name == 'EC-Earth3-i2', TRUE, FALSE) 

  #time for daily data. If recipe asks for time step, we don't need to calculate here
#  fdays <- get_days_dcpp(fyears = fyears, season = season, init = 'Nov', calendar = '360-days')

  
#-------------------------------------------
# Step 1: Load the exp
#-------------------------------------------
  #monthly and daily
  hcst.path <- file.path(archive$src, archive$System[[exp.name]]$src, 
                         '$ensemble$', table, '$var$', grid, version)
  hcst.files <- paste0('$var$_', table, '_*_*_s$syear$-$ensemble$_', grid, '_$chunk$.nc')

  if (store.freq == "monthly_mean") {
  #monthly
  hcst <- Start(dat = file.path(hcst.path, hcst.files),
                var = variable,
                ensemble = member,
                syear = paste0(sdates),
#split_multiselected_dims = T,
                chunk = 'all',
                chunk_depends = 'syear',
                time = indices(time),
                time_across = 'chunk',
                merge_across_dims = TRUE,
                largest_dims_length = need_largest_dims_length,
                latitude = values(list(lats.min, lats.max)),
                latitude_reorder = Sort(decreasing = TRUE),
                longitude = values(list(lons.min, lons.max)),
                longitude_reorder = circularsort,
                transform = regrid_params$fcst.transform,
                transform_extra_cells = 2,
                transform_params = list(grid = regrid_params$fcst.gridtype, #nc file
                                        method = regrid_params$fcst.gridmethod),
                transform_vars = c('latitude', 'longitude'),
                synonims = list(longitude = c('lon', 'longitude'),
                                latitude = c('lat', 'latitude')),
                return_vars = list(latitude = NULL, longitude = NULL, 
                                   time = c('syear', 'chunk')),
                retrieve = T)

  # change syear to c(sday, sweek, syear)
  dim(hcst) <- c(dim(hcst)[1:3], sday = 1, sweek = 1, dim(hcst)[4:7])
  tmp_time_attr <- attr(hcst, 'Variables')$common$time
  dim(attr(hcst, 'Variables')$common$time) <- c(sday = 1, sweek = 1, dim(hcst)[6:7])

  # Change class from startR_array to s2dv_cube
  hcst <- startR_to_s2dv(hcst)

  } else if (store.freq == "daily_mean") {
  # Should be ok to use the call above!!!!!!!!!!
#daily (not done yet)
  data <- Start(dat = file.path(hcst.path, hcst.files),
                var = variable,
                syear = paste0(sdates),
                aux = 'all',
                aux_depends = 'syear',
                latitude = values(list(lats.min, lats.max)),
                latitude_reorder = Sort(decreasing = TRUE),
                longitude = values(list(lons.min, lons.max)),
                longitude_reorder = CircularSort(0, 360),
                fday = indices(fdays),
                ensemble = members,
                synonims = list(fday = c('fday', 'time'),
                               longitude = c('lon', 'longitude'),
                               latitude = c('lat', 'latitude')),
                return_vars = list(latitude = NULL, longitude = NULL,
                                   fday = c('syear', 'aux')),
                retrieve = F)

  }


#-------------------------------------------
# Step 2. Load the reference
#-------------------------------------------
  obs.path <- file.path(archive$src, archive$Reference[[ref.name]]$src, store.freq,
                        paste0(variable, archive$Reference[[ref.name]][[store.freq]][[variable]]))
  obs.files <- paste0('$var$_$file_date$.nc')

  # Get from startR_cube
#  dates <- attr(hcst, 'Variables')$common$time
  # Get from s2dv_cube
  dates <- hcst$Dates$start 
  dates_file <- sapply(dates, format, '%Y%m')
  dim(dates_file) <- dim(dates)

  if (store.freq == "daily_mean") {
#////////////////
# Method 1: use hcst time attr as obs time selector
#////////////////

  # Set hour to 12:00 to ensure correct date retrieval for daily data
  lubridate::hour(dates) <- 12 
  lubridate::minute(dates) <- 00
  # Restore correct dimensions
  dim(dates) <- dim(dates_file)

  obs <- Start(dat = file.path(obs.path, obs.files),
               var = variable,
               file_date = unique(format(dates, '%Y%m')),
               time = dates, # [sday, sweek, syear, time]
               time_across = 'file_date',
               merge_across_dims = TRUE,
               split_multiselected_dims = TRUE,
               latitude = values(list(lats.min, lats.max)),
               latitude_reorder = Sort(decreasing = TRUE), 
               longitude = values(list(lons.min, lons.max)),
               longitude_reorder = circularsort,
               transform = regrid_params$obs.transform,
               transform_extra_cells = 2,
               transform_params = list(grid = regrid_params$obs.gridtype, #nc file
                                       method = regrid_params$obs.gridmethod),
               transform_vars = c('latitude', 'longitude'),
               synonims = list(latitude = c('lat','latitude'), 
                               longitude = c('lon','longitude')),
               return_vars = list(latitude = NULL, longitude = NULL,
                                  time = 'file_date'),
               retrieve = TRUE)

  } else if (store.freq == "monthly_mean") {
#////////////////
# Method 2: reshape hcst time attr's date into an array with time dim then as obs date selector
#////////////////

  obs <- Start(dat = file.path(obs.path, obs.files),
               var = variable,
               file_date = dates_file, #dates_arr, # [sday, sweek, syear, time]
               split_multiselected_dims = TRUE,
               latitude = values(list(lats.min, lats.max)),
               latitude_reorder = Sort(decreasing = TRUE),
               longitude = values(list(lons.min, lons.max)),
               longitude_reorder = circularsort,
               transform = regrid_params$obs.transform,
               transform_extra_cells = 2,
               transform_params = list(grid = regrid_params$obs.gridtype, #nc file
                                       method = regrid_params$obs.gridmethod),
               transform_vars = c('latitude', 'longitude'),
               synonims = list(latitude = c('lat','latitude'),
                               longitude = c('lon','longitude')),
               return_vars = list(latitude = NULL, longitude = NULL,
                                  time = 'file_date'),
               retrieve = TRUE)
  }


dim(attr(obs, 'Variables')$common$time)
# sday sweek syear  time 
#    1     1     2    14 


  # TODO: Reorder obs dims to match hcst dims?
  # Adds ensemble dim to obs (for consistency with hcst/fcst)
  default_dims <- c(dat = 1, var = 1, sweek = 1, 
                    sday = 1, syear = 1, time = 1, 
                    latitude = 1, longitude = 1, ensemble = 1)
  default_dims[names(dim(obs))] <- dim(obs)
  dim(obs) <- default_dims


  # Change class from startR_array to s2dv_cube
  obs <- startR_to_s2dv(obs)



#-------------------------------------------
# Step 3. Verify the consistance btwn hcst and obs
#-------------------------------------------
  # dimension
  if (any(!names(dim(obs$data)) %in% names(dim(hcst$data)))) {
    stop("hcst and obs don't share the same dimension names.")
  } else {
    ens_ind <- which(names(dim(obs$data)) == 'ensemble')
    match_ind <- match(names(dim(obs$data))[-ens_ind], names(dim(hcst$data)))
    if (!all(dim(hcst$data)[match_ind] == dim(obs$data)[-ens_ind])) stop("hcst and obs don't share the same dimension length.")
  }

  # time attribute
  if (!identical(format(hcst$Dates$start, '%Y%m'),
                 format(obs$Dates$start, '%Y%m')))
    stop("hcst and obs don't share the same time.")

  # lat and lon attributes
  if (!identical(as.vector(hcst$lat),
                 as.vector(obs$lat)))
    stop("hcst and obs don't share the same latitude.")
  if (!identical(as.vector(hcst$lon),
                 as.vector(obs$lon)))
    stop("hcst and obs don't share the same longitude.")



  return(list(hcst = hcst, fcst = fcst, obs = obs))
}
